@@ -10,6 +10,7 @@ from typing import (
     Any, Dict, Generator, Iterator, List, Optional, Tuple,
 )
 
+from amundsen_common.utils.atlas import AtlasTableKey
 from atlasclient.client import Atlas
 from pyhocon import ConfigFactory, ConfigTree
 
@@ -18,8 +19,8 @@ from databuilder.extractor.base_extractor import Extractor
 LOGGER = logging.getLogger(__name__)
 
 # custom types
-type_fields_mapping_spec = Dict[str, List[Tuple[str, str, Any, Any]]]
-type_fields_mapping = List[Tuple[str, str, Any, Any]]
+type_fields_mapping_spec = Dict[str, List[Tuple[str, Any, Any, Any]]]
+type_fields_mapping = List[Tuple[str, Any, Any, Any]]
 
 # @todo document classes/methods
 # @todo write tests
@@ -38,6 +39,11 @@ class AtlasSearchDataExtractorHelpers:
         return AtlasSearchDataExtractorHelpers._filter_none(
             [e.get('attributes').get('name') for e in entity_list if e.get('status').lower() == 'active'])
 
+    @staticmethod
+    def get_entity_uri(qualified_name: str, type_name: str) -> str:
+        key = AtlasTableKey(qualified_name, database=type_name)
+        return key.amundsen_key
+
     @staticmethod
     def get_entity_descriptions(entity_list: Optional[List]) -> List:
         entity_list = entity_list or []
@@ -80,6 +86,40 @@ class AtlasSearchDataExtractorHelpers:
 
         return AtlasSearchDataExtractorHelpers.get_display_text(charts)
 
+    @staticmethod
+    def get_table_database(table_key: str) -> str:
+        result = AtlasTableKey(table_key).get_details().get('database', 'hive_table')
+
+        return result
+
+    @staticmethod
+    def get_source_description(parameters: Optional[dict]) -> str:
+        parameters = parameters or dict()
+
+        return parameters.get('sourceDescription', '')
+
+    @staticmethod
+    def get_usage(readers: Optional[List]) -> Tuple[int, int]:
+        readers = readers or []
+
+        score = 0
+        unique = 0
+
+        for reader in readers:
+            reader_status = reader.get('status')
+            entity_status = reader.get('relationshipAttributes', dict()).get('entity', dict()).get('entityStatus', '')
+            relationship_status = reader.get('relationshipAttributes',
+                                             dict()).get('entity',
+                                                         dict()).get('relationshipStatus', '')
+
+            if reader_status == entity_status == relationship_status == 'ACTIVE':
+                score += reader.get('attributes', dict()).get('count', 0)
+
+                if score > 0:
+                    unique += 1
+
+        return score, unique
+
 
 class AtlasSearchDataExtractor(Extractor):
     ATLAS_URL_CONFIG_KEY = 'atlas_url'
@@ -110,15 +150,21 @@ class AtlasSearchDataExtractor(Extractor):
     # es_document field, atlas field path, modification function, default_value
     FIELDS_MAPPING_SPEC: type_fields_mapping_spec = {
         'Table': [
-            ('database', 'typeName', None, None),
-            ('cluster', 'attributes.qualifiedName', lambda x: x.split('@')[-1], None),
-            ('schema', 'relationshipAttributes.db.displayText', None, None),
+            ('database', 'attributes.qualifiedName',
+             lambda x: AtlasSearchDataExtractorHelpers.get_table_database(x), None),
+            ('cluster', 'attributes.qualifiedName',
+             lambda x: AtlasTableKey(x).get_details()['cluster'], None),
+            ('schema', 'attributes.qualifiedName',
+             lambda x: AtlasTableKey(x).get_details()['schema'], None),
             ('name', 'attributes.name', None, None),
-            ('key', 'attributes.qualifiedName', None, None),
+            ('key', ['attributes.qualifiedName', 'typeName'],
+             lambda x, y: AtlasSearchDataExtractorHelpers.get_entity_uri(x, y), None),
             ('description', 'attributes.description', None, None),
             ('last_updated_timestamp', 'updateTime', lambda x: int(x) / 1000, 0),
-            ('total_usage', 'attributes.popularityScore', lambda x: int(x), 0),
-            ('unique_usage', 'attributes.uniqueUsage', lambda x: int(x), 1),
+            ('total_usage', 'relationshipAttributes.readers',
+             lambda x: AtlasSearchDataExtractorHelpers.get_usage(x)[0], 0),
+            ('unique_usage', 'relationshipAttributes.readers',
+             lambda x: AtlasSearchDataExtractorHelpers.get_usage(x)[1], 0),
             ('column_names', 'relationshipAttributes.columns',
              lambda x: AtlasSearchDataExtractorHelpers.get_entity_names(x), []),
             ('column_descriptions', 'relationshipAttributes.columns',
@@ -127,15 +173,19 @@ class AtlasSearchDataExtractor(Extractor):
              lambda x: AtlasSearchDataExtractorHelpers.get_display_text(x), []),
             ('badges', 'classifications',
              lambda x: AtlasSearchDataExtractorHelpers.get_badges_from_classifications(x), []),
-            ('display_name', 'attributes.qualifiedName', lambda x: x.split('@')[0], None),
-            ('schema_description', 'attributes.parameters.sourceDescription', None, None),
+            ('display_name', 'attributes.qualifiedName',
+             lambda x: '.'.join([AtlasTableKey(x).get_details()['schema'], AtlasTableKey(x).get_details()['table']]),
+             None),
+            ('schema_description', 'attributes.parameters',
+             lambda x: AtlasSearchDataExtractorHelpers.get_source_description(x), ''),
             ('programmatic_descriptions', 'attributes.parameters', lambda x: [str(s) for s in list(x.values())], {})
         ],
         'Dashboard': [
             ('group_name', 'relationshipAttributes.group.attributes.name', None, None),
             ('name', 'attributes.name', None, None),
             ('description', 'attributes.description', None, None),
-            ('total_usage', 'attributes.popularityScore', lambda x: int(x), 0),
+            ('total_usage', 'relationshipAttributes.readers',
+             lambda x: AtlasSearchDataExtractorHelpers.get_usage(x)[0], 0),
             ('product', 'attributes.product', None, None),
             ('cluster', 'attributes.cluster', None, None),
             ('group_description', 'relationshipAttributes.group.attributes.description', None, None),
@@ -152,17 +202,35 @@ class AtlasSearchDataExtractor(Extractor):
              lambda x: AtlasSearchDataExtractorHelpers.get_display_text(x), []),
             ('badges', 'classifications',
              lambda x: AtlasSearchDataExtractorHelpers.get_badges_from_classifications(x), [])
+        ],
+        'User': [
+            ('email', 'attributes.qualifiedName', None, ''),
+            ('first_name', 'attributes.first_name', None, ''),
+            ('last_name', 'attributes.last_name', None, ''),
+            ('full_name', 'attributes.full_name', None, ''),
+            ('github_username', 'attributes.github_username', None, ''),
+            ('team_name', 'attributes.team_name', None, ''),
+            ('employee_type', 'attributes.employee_type', None, ''),
+            ('manager_email', 'attributes.manager_email', None, ''),
+            ('slack_id', 'attributes.slack_id', None, ''),
+            ('role_name', 'attributes.role_name', None, ''),
+            ('is_active', 'attributes.is_active', None, ''),
+            ('total_read', 'attributes.total_read', None, ''),
+            ('total_own', 'attributes.total_own', None, ''),
+            ('total_follow', 'attributes.total_follow', None, '')
         ]
     }
 
     ENTITY_MODEL_BY_TYPE = {
         'Table': 'databuilder.models.table_elasticsearch_document.TableESDocument',
-        'Dashboard': 'databuilder.models.dashboard_elasticsearch_document.DashboardESDocument'
+        'Dashboard': 'databuilder.models.dashboard_elasticsearch_document.DashboardESDocument',
+        'User': 'databuilder.models.user_elasticsearch_document.UserESDocument'
     }
 
     REQUIRED_RELATIONSHIPS_BY_TYPE = {
-        'Table': ['columns'],
-        'Dashboard': ['group', 'charts', 'executions', 'queries']
+        'Table': ['columns', 'readers'],
+        'Dashboard': ['group', 'charts', 'executions', 'queries'],
+        'User': []
     }
 
     def init(self, conf: ConfigTree) -> None:
@@ -205,7 +273,7 @@ class AtlasSearchDataExtractor(Extractor):
 
     @property
     def relationships(self) -> Optional[List[str]]:
-        return AtlasSearchDataExtractor.REQUIRED_RELATIONSHIPS_BY_TYPE.get(self.entity_type)
+        return AtlasSearchDataExtractor.REQUIRED_RELATIONSHIPS_BY_TYPE.get(self.entity_type)  # type: ignore
 
     def extract(self) -> Any:
         if not self._extract_iter:
@@ -234,7 +302,7 @@ class AtlasSearchDataExtractor(Extractor):
 
         try:
             return admin_metrics[-1].entity
-        except Exception as e:
+        except Exception:
             return None
 
     def _get_count_of_active_entities(self) -> int:
@@ -339,16 +407,23 @@ class AtlasSearchDataExtractor(Extractor):
                 data = atlas_entity.__dict__['_data']
 
                 for spec in self.field_mappings:
-                    model_field, atlas_field_path, _transform_spec, default_value = spec
+                    model_field, atlas_fields_paths, _transform_spec, default_value = spec
 
-                    atlas_value = reduce(lambda x, y: x.get(y, dict()), atlas_field_path.split('.'),
-                                         data) or default_value
+                    if not isinstance(atlas_fields_paths, list):
+                        atlas_fields_paths = [atlas_fields_paths]
+
+                    atlas_values = []
+                    for atlas_field_path in atlas_fields_paths:
+
+                        atlas_value = reduce(lambda x, y: x.get(y, dict()), atlas_field_path.split('.'),
+                                             data) or default_value
+                        atlas_values.append(atlas_value)
 
                     transform_spec = _transform_spec or (lambda x: x)
 
-                    es_entity_value = transform_spec(atlas_value)
+                    es_entity_value = transform_spec(*atlas_values)
                     model_dict[model_field] = es_entity_value
 
                 yield self.model_class(**model_dict)
             except Exception:
-                LOGGER.warning(f'Error building model object.', exc_info=True)
+                LOGGER.warning('Error building model object.', exc_info=True)
